"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs = require("fs");
const uuid = require("uuid");
const path = require("path");
const base_1 = require("../../test/base");
const egg_controller_1 = require("egg-controller");
const error_1 = require("egg-controller/error");
const __1 = require("..");
const req_test_1 = require("../../test/req_test");
class TestController {
    constructor(ctx) {
        this.config = ctx.app.config.et.sceneTest;
        this.sceneRootDir = path.join(ctx.app.baseDir, this.config.basePath);
        this.mkdir(this.sceneRootDir);
    }
    getSceneTest(id) {
        return new base_1.SceneTestBase(path.join(this.sceneRootDir, id));
    }
    getReqTestPath(sid, rid) {
        return path.join(this.sceneRootDir, sid, rid);
    }
    getReqTest(sid, rid) {
        return new base_1.ReqTestBase(this.getReqTestPath(sid, rid));
    }
    getSceneList() {
        return fs.readdirSync(this.sceneRootDir)
            .map(id => {
            return this.getSceneTest(id).info;
        })
            .filter(info => info);
    }
    getScene(id) {
        const scene = this.getSceneTest(id);
        return Object.assign({}, scene.info, { req: scene.testReq.map(req => req.info) });
    }
    addScene(id, title) {
        const scene = this.getSceneTest(id);
        scene.info = {
            id, title, req: []
        };
        scene.saveInfo();
        return true;
    }
    removeScene(id) {
        this.getSceneTest(id).remove();
    }
    saveScene(id, data) {
        const scene = this.getSceneTest(id);
        scene.info = data;
        scene.saveInfo();
        return this.getScene(data.id);
    }
    getSceneCase(id) {
        return this.getSceneTest(id).testReq.map(req => req.info);
    }
    createSceneReq(id, data = {}) {
        const sceneTest = this.getSceneTest(id);
        data.id = uuid.v4();
        const reqTest = this.getReqTest(id, data.id);
        reqTest.info = data;
        reqTest.saveInfo();
        if (!sceneTest.info.req) {
            sceneTest.info.req = [];
        }
        sceneTest.info.req.push(data.id);
        sceneTest.saveInfo();
        return this.getScene(id);
    }
    removeSceneReq(id, rid) {
        this.getReqTest(id, rid).remove();
    }
    createSceneReqByTracking(id, tid, title) {
        const trackingData = __1.getTrackingMetadata().find(t => t.id === tid);
        if (!trackingData) {
            throw new error_1.BadRequestError('未找到跟踪数据');
        }
        const sceneTest = this.getSceneTest(id);
        if (!sceneTest) {
            throw new error_1.NotFoundError('未找到测试场景');
        }
        // 数据修剪
        const header = Object.assign({}, trackingData.extInfo.header);
        delete header['content-length'];
        const reqTest = sceneTest.createReqTest({
            title: title || (trackingData.extInfo.route || {}).name,
            stTime: trackingData.time,
            method: trackingData.extInfo.method,
            url: trackingData.extInfo.originalUrl,
            header: header,
            body: trackingData.extInfo.body,
            judge: '',
            result: trackingData.extInfo.result,
            mockBlackList: [],
            mockWhiteList: []
        });
        // 整理mock数据
        const calls = this.getEdgeCall(trackingData.calls)
            .filter(call => {
            // 暂不支持自定义类作为mock数据
            if ([
                'Object', 'string', 'boolean', 'number', 'null', 'Array', 'Date', 'undefined', 'Error',
                /** mysql 返回数据类型 */
                'OkPacket', 'RowDataPacket',
            ].some(s => s === call.retType)) {
                return true;
            }
            return false;
        });
        const mockData = reqTest.info.mockData = {};
        calls.forEach(call => {
            if (!mockData[call.meta.clsName]) {
                mockData[call.meta.clsName] = {};
            }
            const clsMockData = mockData[call.meta.clsName];
            if (!clsMockData[call.method]) {
                clsMockData[call.method] = [];
            }
            clsMockData[call.method].push({
                args: call.args,
                isErr: call.isErr,
                retType: call.retType,
                ret: call.ret,
            });
        });
        reqTest.saveMockData();
        return reqTest.info;
    }
    onlineTest(id, rid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const req = new req_test_1.ReqTest(this.getReqTestPath(id, rid));
            if (!req.info) {
                throw new error_1.NotFoundError();
            }
            return yield req.runTest();
        });
    }
    updateTest(id, rid) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const req = new req_test_1.ReqTest(this.getReqTestPath(id, rid));
            if (!req.info) {
                throw new error_1.NotFoundError();
            }
            return yield req.setMode('update').runTest();
        });
    }
    mkdir(dirPath, onErr) {
        try {
            if (!fs.existsSync(dirPath)) {
                fs.mkdirSync(dirPath);
            }
        }
        catch (error) {
            onErr && onErr(error);
        }
    }
    getEdgeCall(callData, dataList = []) {
        callData.forEach(call => {
            const clsMethod = `${call.meta.clsName}.${call.method}`;
            if (call.calls.length) {
                // 白名单
                if (this.config.trackingSaveWhiteList.some(regex => regex.test(clsMethod))) {
                    dataList.push(call);
                }
                this.getEdgeCall(call.calls, dataList);
            }
            else if (
            // 黑名单
            this.config.trackingSaveBlackList.every(regex => !regex.test(clsMethod))) {
                dataList.push(call);
            }
        });
        return dataList;
    }
}
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene`,
        name: '获取测试场景', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", []),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "getSceneList", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id`,
        name: '获取测试场景详情', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "getScene", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene`,
        method: 'post', name: '添加测试场景', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "addScene", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id`,
        method: 'delete', name: '删除测试场景', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "removeScene", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id`,
        method: 'put', name: '更新测试场景', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "saveScene", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/req`,
        name: '获取测试场景请求列表', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "getSceneCase", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id/req`,
        method: 'post', name: '添加测试场景请求', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Object]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "createSceneReq", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id/req/:rid`,
        method: 'delete', name: '删除测试场景请求', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "removeSceneReq", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id/reqByTracking`,
        method: 'post', name: '使用跟踪数据添加测试场景请求', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, Number, String]),
    tslib_1.__metadata("design:returntype", void 0)
], TestController.prototype, "createSceneReqByTracking", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id/req/:rid`,
        method: 'get', name: '在线使用测试用例', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", Promise)
], TestController.prototype, "onlineTest", null);
tslib_1.__decorate([
    egg_controller_1.route({
        url: config => `${config.et.metaPath}/test/scene/:id/req/:rid`,
        method: 'put', name: '更新使用测试用例', noTracking: true
    }),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, String]),
    tslib_1.__metadata("design:returntype", Promise)
], TestController.prototype, "updateTest", null);
exports.TestController = TestController;
