"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("colors");
const fs = require("fs");
const Diff = require("diff");
const path = require("path");
const rimraf = require("rimraf");
const uuid = require("uuid");
const tracking_1 = require("../../tracking");
const utils_1 = require("power-di/utils");
const di_1 = require("../../di");
exports.INFO_FILE_NAME = 'info.json';
exports.MOCK_DATA_DIR = 'mock-data';
class TestBase {
    constructor(dir) {
        this.dir = dir;
        this.infoPath = path.join(this.dir, exports.INFO_FILE_NAME);
    }
    mkdir(dir) {
        if (!fs.existsSync(dir)) {
            this.mkdir(path.dirname(dir));
            fs.mkdirSync(dir);
        }
    }
    readFile(filePath) {
        return JSON.parse(fs.readFileSync(filePath, 'utf-8'));
    }
    writeFile(filePath, data) {
        fs.writeFileSync(filePath, JSON.stringify(data, null, 2));
    }
    saveInfo(data) {
        this.mkdir(this.dir);
        delete data.id;
        this.writeFile(this.infoPath, data);
    }
    loadInfo() {
        if (fs.existsSync(this.infoPath)) {
            const info = this.readFile(this.infoPath);
            info.id = path.parse(this.dir).name;
            return info;
        }
    }
    remove() {
        rimraf.sync(this.dir);
    }
}
exports.TestBase = TestBase;
const OriginDate = Date;
class ReqTestBase extends TestBase {
    constructor(dir) {
        super(dir);
        this.mode = 'normal';
        this.tmpMockData = {};
        this.mockFunc = (inst) => {
            const self = this;
            const { mockData } = this.info;
            const runMode = this.mode;
            // 类级 Mock 数据
            const clsType = utils_1.getGlobalType(inst.constructor);
            const superClasses = utils_1.getSuperClassInfo(inst.constructor);
            const mockClsNames = [clsType].concat(superClasses.map(ti => ti.type)).filter(cn => !!mockData[cn]);
            const clsMockData = mockClsNames.length && mockClsNames.map(mockClsName => mockData[mockClsName]);
            return new Proxy(inst, {
                get(target, property) {
                    const originProp = target[property];
                    if (typeof property !== 'string' ||
                        property === 'constructor' ||
                        !clsMockData ||
                        clsMockData.every(mockData => mockData[property] === undefined)) {
                        return originProp;
                    }
                    // 方法级 Mock 数据
                    const clsMockDataIndex = clsMockData.findIndex(mockData => mockData[property] !== undefined);
                    const mockClsName = mockClsNames[clsMockDataIndex];
                    const methodMockData = clsMockData[clsMockDataIndex][property];
                    if (!self.tmpMockData[mockClsName]) {
                        self.tmpMockData[mockClsName] = {};
                    }
                    const tmpMockData = self.tmpMockData[mockClsName];
                    function updateMockData(index = -1, deleteCount = 0) {
                        function getResult(ctx) {
                            if (index === -1) {
                                methodMockData.push(Object.assign({ args: ctx.args }, tracking_1.getResultFromCtx(ctx)));
                            }
                            else {
                                methodMockData.splice(index, deleteCount, Object.assign({ args: ctx.args }, tracking_1.getResultFromCtx(ctx)));
                            }
                        }
                        return di_1.funcWrapper({
                            after: getResult,
                            error: getResult,
                        }, originProp);
                    }
                    if (runMode === 'update') {
                        methodMockData.splice(0);
                        updateMockData();
                    }
                    else if (runMode === 'normal' || runMode === 'fix') {
                        if (!tmpMockData[property]) {
                            tmpMockData[property] = [...methodMockData];
                        }
                        const thisMockData = tmpMockData[property];
                        return function (...args) {
                            const acArgs = JSON.stringify(args);
                            if (!thisMockData.length) {
                                if (runMode === 'fix') {
                                    return updateMockData().bind(target)(...args);
                                }
                                else {
                                    throw new Error(`Mock数据数量不匹配，type:[${clsType}(${mockClsName})] method:[${property}] 剩余:[${thisMockData.length}]\n`);
                                }
                            }
                            let callDataIndex = thisMockData.findIndex(d => acArgs === JSON.stringify(d.args));
                            // FIXME 绕过修改数据时修改时间不匹配问题
                            if (callDataIndex < 0 &&
                                args.length === 2 &&
                                typeof args[0] === 'string' &&
                                ['update', 'insert'].some(s => args[0].toLowerCase().startsWith(s)) &&
                                args[0].toLowerCase().indexOf('gmt_modified') > 0) {
                                callDataIndex = thisMockData.findIndex(m => {
                                    if (m.args[0] !== args[0])
                                        return false;
                                    return args[1].filter((arg, i) => {
                                        const mockArg = m.args[1][i];
                                        // 忽略 null undefined 判断
                                        if ((arg === undefined || arg === null) && (mockArg === undefined || mockArg === null)) {
                                            return false;
                                        }
                                        // 忽略 Date
                                        if (arg instanceof Date && typeof mockArg === 'string' && new Date(mockArg).toString() !== 'Invalid Date') {
                                            return false;
                                        }
                                        return arg !== mockArg;
                                    }).length === 0;
                                });
                            }
                            if (callDataIndex < 0) {
                                if (runMode === 'fix') {
                                    const readIndex = methodMockData.indexOf(thisMockData[0]);
                                    const updateIndex = methodMockData.findIndex((m, i) => {
                                        return i >= readIndex && m.args[0] === args[0];
                                    });
                                    return updateMockData(updateIndex === -1 ? readIndex : updateIndex, updateIndex === -1 ? 0 : 1).bind(target)(...args);
                                }
                                else {
                                    let targetMockData = thisMockData[0];
                                    if (acArgs.length < 5000) {
                                        const sorts = thisMockData.map(t => {
                                            return Diff.diffWords(acArgs, JSON.stringify(t.args)).length;
                                        });
                                        let minIndex = 0, minScore = sorts[0];
                                        sorts.forEach((s, i) => {
                                            if (s < minScore) {
                                                minIndex = i;
                                            }
                                        });
                                        console.log('[Test Request Id]', self.info.id);
                                        console.log('[by diff change count]', sorts[minIndex], self.info.id);
                                        if (sorts[minIndex] > 8) {
                                            console.log('差异过大，未找到匹配项');
                                            targetMockData = undefined;
                                        }
                                        else {
                                            targetMockData = thisMockData[minIndex];
                                        }
                                    }
                                    function hasDiff(part) {
                                        return part && (part.added || part.removed);
                                    }
                                    if (targetMockData) {
                                        targetMockData.args.map((arg, i) => ({ i, arg }))
                                            .filter(data => JSON.stringify(data.arg) !== JSON.stringify(args[data.i]))
                                            .forEach(data => {
                                            console.log(`[参数 {${data.i}} 不匹配]:`);
                                            let diffResult;
                                            if (typeof data.arg === 'string' && typeof args[data.i] === 'string') {
                                                diffResult = Diff.diffWords(data.arg, args[data.i]);
                                            }
                                            else {
                                                diffResult = Diff.diffLines(JSON.stringify(data.arg, null, 2), JSON.stringify(args[data.i], null, 2));
                                            }
                                            diffResult.filter((part, i, parts) => {
                                                return hasDiff(part) ||
                                                    hasDiff(parts[i - 1]) ||
                                                    hasDiff(parts[i + 1]);
                                            }).forEach(part => {
                                                let color = part.added ? 'green' :
                                                    part.removed ? 'red' : 'grey';
                                                process.stderr.write(part.value[color]);
                                            });
                                            console.log();
                                        });
                                    }
                                    console.log('[实际入参]', JSON.stringify(args));
                                    throw new Error(`Mock数据参数不匹配，type:[${clsType}(${mockClsName})] method:[${property}] 剩余:[${thisMockData.length}]\n`);
                                }
                            }
                            const callData = thisMockData.splice(callDataIndex, 1)[0];
                            if (callData.isErr) {
                                throw new Error(callData.err);
                            }
                            else if (callData.retType === 'Date') {
                                return new Date(callData.ret);
                            }
                            else {
                                return callData.ret;
                            }
                        };
                    }
                }
            });
        };
        this.mockDir = path.join(this.dir, exports.MOCK_DATA_DIR);
        this.info = this.loadInfo();
        if (this.info) {
            this.info.mockData = {};
            this.loadMockData();
        }
    }
    setMode(mode) {
        this.mode = mode;
        return this;
    }
    loadMockData() {
        if (fs.existsSync(this.mockDir)) {
            fs.readdirSync(this.mockDir).forEach(clsName => {
                const clsMockData = {};
                this.info.mockData[clsName] = clsMockData;
                fs.readdirSync(path.join(this.mockDir, clsName)).forEach(methodFile => {
                    clsMockData[path.parse(methodFile).name] = JSON.parse(fs.readFileSync(path.join(this.mockDir, clsName, methodFile), 'utf-8'));
                });
            });
        }
    }
    saveMockData() {
        if (this.info) {
            rimraf.sync(this.mockDir);
            Object.keys(this.info.mockData).forEach(clsName => {
                const clsMockData = this.info.mockData[clsName];
                const clsDir = path.join(this.mockDir, clsName);
                this.mkdir(clsDir);
                Object.keys(clsMockData).forEach(method => {
                    this.writeFile(path.join(clsDir, `${method}.json`), clsMockData[method]);
                });
            });
        }
    }
    startMockData() {
        this.tmpMockData = {};
        this.startMockTime = +new OriginDate();
        const getNow = () => {
            const between = OriginDate.now() - this.startMockTime;
            return this.info.stTime + between;
        };
        Date = new Proxy(OriginDate, {
            get: (target, p) => {
                if (p === 'now') {
                    return getNow;
                }
                if (p === 'constructor') {
                    return function (...args) {
                        if (args.length) {
                            return new OriginDate(...args);
                        }
                        return new OriginDate(getNow());
                    };
                }
                return target[p];
            }
        });
        di_1.setCreateInstanceHook(this.mockFunc);
    }
    stopMockData() {
        di_1.removeCreateInstanceHook(this.mockFunc);
        Date = OriginDate;
        if (this.mode === 'update' || this.mode === 'fix') {
            this.saveMockData();
        }
    }
    saveInfo() {
        if (this.info) {
            const data = Object.assign({}, this.info);
            delete data.mockData;
            super.saveInfo(data);
        }
    }
    addMockData(clsName, methodName, data) {
        const mockData = this.info.mockData;
        if (!mockData[clsName]) {
            mockData[clsName] = {};
        }
        const clsMockData = mockData[clsName];
        if (!clsMockData[methodName]) {
            clsMockData[methodName] = [];
        }
        clsMockData[methodName].push(data);
    }
}
exports.ReqTestBase = ReqTestBase;
class SceneTestBase extends TestBase {
    constructor(dir) {
        super(dir);
        this.info = this.loadInfo();
        if (this.info) {
            this.info.req = this.info.req || [];
            this.info.req = Array.from(new Set(this.info.req.concat(fs.readdirSync(dir) || [])))
                .filter(reqDir => fs.existsSync(path.join(dir, reqDir)) &&
                fs.statSync(path.join(dir, reqDir)).isDirectory());
            this.loadReqTest();
            this._testReq = this._testReq.filter(req => req.info);
            this.info.req = this._testReq.map(req => req.info.id);
            this.saveInfo();
        }
    }
    get testReq() { return this._testReq; }
    setReqRunMode(mode) {
        this._testReq.forEach(req => req.setMode(mode));
        return this;
    }
    loadReqTest() {
        this._testReq = this.info.req.map(reqId => {
            return new ReqTestBase(path.join(this.dir, reqId));
        });
    }
    saveInfo() {
        if (this.info) {
            super.saveInfo(Object.assign({}, this.info));
            this._testReq
                .filter(req => this.info.req.every(r => r !== req.info.id))
                .forEach(req => req.remove());
            this.loadReqTest();
        }
    }
    createReqTest(info) {
        const id = uuid.v4();
        const reqTest = new ReqTestBase(path.join(this.dir, id));
        reqTest.info = info;
        reqTest.saveInfo();
        this.info.req.push(id);
        this.loadReqTest();
        this.saveInfo();
        return reqTest;
    }
}
exports.SceneTestBase = SceneTestBase;
